/// \file extmil1553ud_library.c
/// \brief библиотка взаимодействия
/// реалитзация функций
#include "mil1553udext_library.h"
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>


#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#else
    #include <pthread.h>
    #include <unistd.h>
    #include <unistd.h>
    #include <sys/ioctl.h>  /* ioctl */
    #include <sys/mman.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <arpa/inet.h>
    #include <netdb.h>
#endif

#include "mil1553udext_eth_ioctl.h"
#include "mil1553udext_ioctl_wrappers.h"

/// \brief паттерн имени девайса
#define MIL1553_STR_CH0_PATTERN "-ch-0"
/// \brief паттерн имени канала
#define MIL1553_STR_DEV_PATTERN "mil1553dev"
/// \brief каталог с каналами
#define MIL1553_STR_DEV "/dev"

extern inline uint32_t EXT_INSTRUCTION(uint32_t instruction, uint32_t condition, uint32_t parameter);

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. отладочные IOCTL функции
/// /// /// /// /// /// /// /// /// /// /// ///

// #define DEBUG_MIL1553_LIB


/// \brief Таймаут сетевых операций (поле структуры: секунды)
#define SOCKET_IO_TIMEOUT_SEC 15
/// \brief Таймаут сетевых операций (поле структуры: микросекунды)
#define SOCKET_IO_TIMEOUT_USEC 0

#ifdef _WIN32
int initWSA()
{
    WSADATA wsaData = {0};

    // Initialize Winsock
    int iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
    if (iResult != 0) {
        printf("WSAStartup failed: %d\n", iResult);
        return 1;
    }

    return iResult;
}
#endif

Mil1553ChannelFD extmil1553_open(const char *address, uint16_t port, uint16_t channel)
{
    Mil1553ChannelFD result;
    struct sockaddr_in addr; // структура с адресом
    int ret, sockopt_idx, keepalive_enabled = 1;
    struct timeval sock_timeout = {
        .tv_sec = SOCKET_IO_TIMEOUT_SEC,
                .tv_usec = SOCKET_IO_TIMEOUT_USEC
    };

#ifdef _WIN32
    static bool wsaInit = false;
    if(!wsaInit)
        initWSA();
#endif

#ifdef DEBUG_MIL1553_LIB
    static const char *opts[] = { "SO_RCVTIMEO", "SO_SNDTIMEO" };
#endif

    result.channel = channel;
    result.socketFD = socket(AF_INET, SOCK_STREAM, 0); // создание TCP-сокета

    if(result.socketFD < 0)
    {
        perror("socket");
        return result;
    }

    for(sockopt_idx = 0; sockopt_idx < 2; ++sockopt_idx)
    {
        if ( (ret = setsockopt(result.socketFD, SOL_SOCKET, sockopt_idx ? SO_RCVTIMEO : SO_SNDTIMEO,
                               (const char *) &sock_timeout, sizeof(sock_timeout))) < 0)
        {
#ifdef DEBUG_MIL1553_LIB
            printf("client: setsockopt failed (%s, %d)!\n", opts[sockopt_idx], ret);
#endif
            goto err;
        }
    }

    if ( (ret = setsockopt(result.socketFD, SOL_SOCKET, SO_KEEPALIVE,
                           (const char *) &keepalive_enabled, sizeof(int))) < 0)
    {
#ifdef DEBUG_MIL1553_LIB
        printf("client: setsockopt failed SO_KEEPALIVE, %d!\n", ret);
#endif
        goto err;
    }

    // Указываем параметры сервера
    addr.sin_family = AF_INET; // домен Internet
    addr.sin_port = htons(port); // порт
    addr.sin_addr.s_addr = inet_addr(address);
    if(addr.sin_addr.s_addr == INADDR_NONE)
    {
        perror("bad inet_addr");
        goto err;
    }

    if(connect(result.socketFD, (struct sockaddr *)&addr, sizeof(addr)) < 0) // установка соединения с сервером
    {
        perror("connect");
        goto err;
    }

    return result;

err:
    extmil1553_close(result);
    result.socketFD = -1;
    return result;
}

int extmil1553_close(Mil1553ChannelFD cfd)
{
#ifdef _WIN32
    return closesocket(cfd.socketFD);
#else
    return close(cfd.socketFD);
#endif
}


unsigned int extioctl_IOCTL_READ_BLOCKS_RAW_DMA(Mil1553ChannelFD cfd, DMA_READ_BLOCK *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_READ_BLOCKS_RAW_DMA, block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_READ_BLOCKS_RAW_DMA ret = %d; countBlocks= %d \n", fd,
           ret, block->countBlocks);
#endif
    return ret;
}

unsigned int extioctl_IOCTL_READ_BLOCKS_RAW_DMA_FIX_BLOCKS(Mil1553ChannelFD cfd, MIL1553_DMA_FIX_BLOCK *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT, block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_READ_BLOCKS_RAW_DMA_FIX_COUNT ret = %d; countBlocks= %d, countBlocks= %d, nextBlocks= %d \n", fd,
           ret, block->mallocedBlocks, block->currentBlocks, block->nextBlocks);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_WRITE_REG(Mil1553ChannelFD cfd, SADDR_DATA *reg)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_WRITE_REG, reg);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_WRITE_REG ret = %d; adr= 0x%x, data= 0x%x \n", fd, ret,
           reg->daddr, reg->data);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_READ_REG(Mil1553ChannelFD cfd, SADDR_DATA *reg)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_READ_REG, reg);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_READ_REG ret = %d; adr= 0x%x, data= 0x%x \n", fd, ret,
           reg->daddr, reg->data);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_WRITE_REG_BIT_MASK(Mil1553ChannelFD cfd, SADDR_DATA_BIT_MASK *reg)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_WRITE_REG_BIT_MASK, reg);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_WRITE_REG_BIT_MASK ret = %d; adr= 0x%x, data= 0x%x, mask= 0x%x \n", fd, ret,
           reg->daddr, reg->data, reg->mask);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_VERSION(Mil1553ChannelFD cfd, VERSION *info)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_VERSION, info);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_VERSION ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_VERSION_DRIVER(Mil1553ChannelFD cfd, unsigned int *version)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_VERSION_DRIVER, version);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_VERSION_DRIVER ret = %d \n", fd, ret);

#endif
    return ret;
}


unsigned int extioctl_IOCTL_CLEAR_DMA(Mil1553ChannelFD cfd)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_CLEAR_DMA);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_CLEAR_DMA ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_GET_COUNT_CHANNELS(Mil1553ChannelFD cfd)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_EXT_GET_COUNT_CHANNELS);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: GET_COUNT_CHANNELS ret = %d \n", fd, ret);

#endif
    return ret;
}


unsigned int extioctl_IOCTL_ENABLE_DMA(Mil1553ChannelFD cfd)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_ENABLE_DMA);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_ENABLE_DMA ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_DISABLE_DMA(Mil1553ChannelFD cfd)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_DISABLE_DMA);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_DISABLE_DMA ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_GET_NBLOCK_RAW_DMA(Mil1553ChannelFD cfd, unsigned int *countBlocks)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_GET_NBLOCK_RAW_DMA, countBlocks);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_GET_NBLOCK_RAW_DMA ret = %d, countBlocks = %d \n", fd, ret, *countBlocks);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_MANAGE_INTERRUPT(Mil1553ChannelFD cfd, INTERRUPT_MAN *man)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_MANAGE_INTERRUPT, man);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_MANAGE_INTERRUPT ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_MAN_IRQ_SUBADDRESS_TR(Mil1553ChannelFD cfd, INTERRUPT_SUBADDRESS *man)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_MAN_IRQ_SUBADDRESS_TR, man);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_MAN_IRQ_SUBADDRESS_TR ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_MAN_IRQ_SUBADDRESS_RCV(Mil1553ChannelFD cfd, INTERRUPT_SUBADDRESS *man)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_MAN_IRQ_SUBADDRESS_RCV, man);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_MAN_IRQ_SUBADDRESS_RCV ret = %d \n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SWITCH_MODE(Mil1553ChannelFD cfd, unsigned int *role)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SWITCH_MODE, role);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SWITCH_MODE ret = %d role = %d \n", fd, ret, *role);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SWITCH_BUS(Mil1553ChannelFD cfd, unsigned int *bus)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SWITCH_BUS, bus);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SWITCH_BUS ret = %d role = %d \n", fd, ret, *bus);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_DEV_CHANNEL_WORK(Mil1553ChannelFD cfd, unsigned int *op)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_DEV_CHANNEL_WORK, op);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_DEV_CHANNEL_WORK ret = %d role = %d \n", fd, ret, *op);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SET_RT_TR_MODE(Mil1553ChannelFD cfd, RT_TR_MODE *rt_tr_mode)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SET_RT_TR_MODE, rt_tr_mode);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SET_RT_TR_MODE ret = %d\n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SET_RT_TR_BUF_READY(Mil1553ChannelFD cfd, RT_TR_MODE_BUF *rdBuf)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SET_RT_TR_BUF_READY, rdBuf);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SET_RT_TR_BUF_READY ret = %d\n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_RT_BUF_MAN_EN(Mil1553ChannelFD cfd, RT_BUF_MAN_EN *rtBufMan)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_RT_BUF_MAN_EN, rtBufMan);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_RT_BUF_MAN_EN ret = %d, %x : %x : %x\n", fd, ret, rtBufMan->direction, rtBufMan->action_mask, rtBufMan->subaddress_mask);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SET_ADDRESS(Mil1553ChannelFD cfd, unsigned char *address)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SET_ADDRESS, address);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SET_ADDRESS ret = %d, address = %d\n", fd, ret, *address);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI(Mil1553ChannelFD cfd, TIMER_TIMEOUT *tmr)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SET_TIMER_RTBM_CONF_REG_PCI, tmr);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SET_TIMER_RTBM_CONF_REG_PCI ret = %d\n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_SET_TIMER_BC_CONF_REG_PCI(Mil1553ChannelFD cfd, TIMER_TIMEOUT *tmr)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_SET_TIMER_BC_CONF_REG_PCI , tmr);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_SET_TIMER_BC_CONF_REG_PCI  ret = %d\n", fd, ret);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_WR_BLOCK_BUF_SUBADR(Mil1553ChannelFD cfd, RT_TR_BUF_SUBADDR *buf)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_WR_BLOCK_BUF_SUBADR , buf);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_WR_BLOCK_BUF_SUBADR  ret = %d, subaddress = %d\n", fd, ret, buf->subaddress);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_RD_BLOCK_BUF_SUBADR(Mil1553ChannelFD cfd, RT_TR_BUF_SUBADDR *buf)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_RD_BLOCK_BUF_SUBADR , buf);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_RD_BLOCK_BUF_SUBADR  ret = %d, subaddress = %d\n", fd, ret, buf->subaddress);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_WRITE_BC_RAM(Mil1553ChannelFD cfd, BC_RAM_BLOCK *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_WRITE_BC_RAM , block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_WRITE_BC_RAM  ret = %d, type = %d, shift = %d, length = %d\n", fd, ret, block->type, block->shift, block->length);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_READ_BC_RAM(Mil1553ChannelFD cfd, BC_RAM_BLOCK *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_READ_BC_RAM , block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_READ_BC_RAM  ret = %d, type = %d, shift = %d, length = %d\n", fd, ret, block->type, block->shift, block->length);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_GET_COUNT_INTERRUPT_BLOCKS(Mil1553ChannelFD cfd, unsigned int *countBlocks)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_GET_COUNT_INTERRUPT_BLOCKS, countBlocks);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_GET_COUNT_INTERRUPT_BLOCKS  ret = %d, countBlocks = %d\n", fd, ret, *countBlocks);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_READ_INTERRUPT_BLOCKS(Mil1553ChannelFD cfd, INTERRUPT_BLOCK_BUFFER *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_READ_INTERRUPT_BLOCKS, block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_READ_INTERRUPT_BLOCKS  ret = %d, count_blocks = %d\n", fd, ret, block->count_blocks);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_GET_RCV_SUBADR_DATA(Mil1553ChannelFD cfd, SUBA_DATA_BLOCK *block)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_GET_RCV_SUBADR_DATA, block);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_GET_RCV_SUBADR_DATA  ret = %d, sa = %d\n", fd, ret, block->sa);

#endif
    return ret;
}

unsigned int extioctl_IOCTL_GET_NUMBER_CH(Mil1553ChannelFD cfd, unsigned int *num)
{
    unsigned int ret = ext_ioctl_eth_handler(cfd, IOCTL_GET_NUMBER_CH, num);
#ifdef DEBUG_MIL1553_LIB
    printf("fd= %d, ch= %d: IOCTL_GET_NUMBER_CH  ret = %d, num = %d\n", fd, ret, *num);

#endif
    return ret;
}

/// /// /// /// /// /// /// /// /// /// /// ///
/// \remark МАКРОБИБЛИОТЕКА MIL1553. ФУНКЦИИ.
/// /// /// /// /// /// /// /// /// /// /// ///

unsigned int extmil1553_readDma(Mil1553ChannelFD cfd, struct MIL1553_DMA_CONTAINER_BLOCKS *container)
{
    DMA_READ_BLOCK block;
    unsigned int ret = GOOD_CODE;
    unsigned int countReadyReadBlocks = 0;
    countReadyReadBlocks = extmil1553ud_getCountDmaBlocksReadyRead(cfd, &ret);
    container->mCount = 0;
    if (ret) {
        goto END;
    }
    if (countReadyReadBlocks == 0) {
        goto END;
    }
    if (countReadyReadBlocks >= MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE)
        container->mCount = MIL1553_DMA_CONTAINER_BLOCKS_MAX_SIZE;
    else
        container->mCount = countReadyReadBlocks;
    block.countBlocks = container->mCount;
    block.length = block.countBlocks * MIL1553_DMA_BLOCK_SIZE;
    block.data = (unsigned char *)malloc(block.length);
    ret = WRAPPER_IOCTL_READ_BLOCKS_RAW_DMA(cfd, &block);
    if (ret) {
        goto FREE;
    }
    memcpy(container->mBlocks, block.data, block.length);
FREE:
    free(block.data);
END:
    return ret;
}

unsigned int extmil1553_readDmaFixBlocks(Mil1553ChannelFD cfd, MIL1553_DMA_FIX_BLOCK *container)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_READ_BLOCKS_RAW_DMA_FIX_BLOCKS(cfd, container);
    return ret;
}

unsigned int extmil1553_manChannel(Mil1553ChannelFD cfd, unsigned int op)
{
    unsigned int ret = GOOD_CODE;
    SADDR_DATA_BIT_MASK reg;
    unsigned int numChannel = cfd.channel;

    reg.daddr = MIL_REG(numChannel, CTRL_REG_PCI_CH);
    reg.mask = (1 << 23);

    if (MIL_DEV_CHANNEL_ON == op) {
        reg.data = (1 << 23);
    } else {
        reg.data = (0 << 23);
    }
    ret = extmil1553_modifyReg(cfd, &reg);
    return ret;
}

unsigned int extmil1553_setRole(Mil1553ChannelFD cfd, unsigned int role)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_SWITCH_MODE(cfd, &role);
    return ret;
}

int extmil1553_getCountChannels(Mil1553ChannelFD cfd)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_GET_COUNT_CHANNELS(cfd);
    return ret;
}

unsigned int extmil1553_writeReg(Mil1553ChannelFD cfd, SADDR_DATA *reg)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_WRITE_REG(cfd, reg);
    return ret;
}

unsigned int extmil1553_readReg(Mil1553ChannelFD cfd, SADDR_DATA *reg)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_READ_REG(cfd, reg);
    return ret;
}

unsigned int extmil1553_modifyReg(Mil1553ChannelFD cfd, SADDR_DATA_BIT_MASK *reg)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_WRITE_REG_BIT_MASK(cfd, reg);
    return ret;
}

unsigned int extmil1553_getDeviceInfo(Mil1553ChannelFD cfd, VERSION *info)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_VERSION(cfd, info);
    return ret;
}

unsigned int extmil1553_getDriverVersion(Mil1553ChannelFD cfd, unsigned int *version)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_VERSION_DRIVER(cfd, version);
    return ret;
}

unsigned int extmil1553_clearDma(Mil1553ChannelFD cfd)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_CLEAR_DMA(cfd);
    return ret;
}

unsigned int extmil1553_manDma(Mil1553ChannelFD cfd, int op)
{
    unsigned int ret = GOOD_CODE;
    if (op) {
        ret = WRAPPER_IOCTL_ENABLE_DMA(cfd);
    } else {
        ret = WRAPPER_IOCTL_DISABLE_DMA(cfd);
    }
    return ret;
}

unsigned int extmil1553_switchBus(Mil1553ChannelFD cfd, unsigned int bus)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_SWITCH_BUS(cfd, &bus);
    return ret;
}

#define MIL1553_BC_RAM_WRITE_SIZE_BLOCK 128

unsigned int extmil1553_bc_writeDataToRam(Mil1553ChannelFD cfd, enum EMil1553BcTypeDataRam type, unsigned int *data, int length)
{
    BC_RAM_BLOCK *blockWrite;
    BC_RAM_BLOCK *blockRead;
    int ret = GOOD_CODE;
    unsigned int index = 0;
    unsigned int i = 0;
    unsigned int countBlocks = 0;
    unsigned int sizeTailBlock = 0;
    if ((length > MIL1553_BC_MAX_SIZE_RAM_NONDATA) &&
            (type != Mil1553BcTypeData)) {
        ret = MIL1553_BC_WRT_DATA_RAM_ERROR_MAX_LEN;
    } else if ((length > MIL1553_BC_MAX_SIZE_RAM_DATA) &&
               (type == Mil1553BcTypeData)) {
        ret = MIL1553_BC_WRT_DATA_RAM_ERROR_MAX_LEN;
    }
    if (ret != GOOD_CODE) return ret;
    blockWrite =
            mil1553ud_newBcRamBlock(type, MIL1553_BC_RAM_WRITE_SIZE_BLOCK, 0);
    blockRead = mil1553ud_newBcRamBlock(type, MIL1553_BC_RAM_WRITE_SIZE_BLOCK, 0);
    countBlocks = length / MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
    sizeTailBlock = length % MIL1553_BC_RAM_WRITE_SIZE_BLOCK;

    for (index = 0; index < countBlocks; index++) {
        blockWrite->shift = index * MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
        blockRead->shift = blockWrite->shift;
        memcpy(blockWrite->dwords, data + blockWrite->shift,
               MIL1553_BC_RAM_WRITE_SIZE_BLOCK * sizeof(unsigned int));
        // записываем микропрограмму
        ret = WRAPPER_IOCTL_WRITE_BC_RAM(cfd, blockWrite);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_WRT;
            goto END;
        }
        ret = WRAPPER_IOCTL_READ_BC_RAM(cfd, blockRead);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_RD;
            goto END;
        }
        // пословное сравнение
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            unsigned int valWrt = blockWrite->dwords[i], valRd = blockRead->dwords[i];
            if (valWrt != valRd) {
                ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_CHECK;
                goto END;
            }
        }
    }
    if (sizeTailBlock) {
        blockWrite->shift = index * MIL1553_BC_RAM_WRITE_SIZE_BLOCK;
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            if (i >= sizeTailBlock)
                blockWrite->dwords[i] = 0;
            else
                blockWrite->dwords[i] = data[blockWrite->shift + i];
        }
        // записываем микропрограмму
        ret = WRAPPER_IOCTL_WRITE_BC_RAM(cfd, blockWrite);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_WRT;
            goto END;
        }
        ret = WRAPPER_IOCTL_READ_BC_RAM(cfd, blockRead);
        if (ret) {
            ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_RD;
            goto END;
        }
        // пословное сравнение
        for (i = 0; i < MIL1553_BC_RAM_WRITE_SIZE_BLOCK; i++) {
            unsigned int valWrt = blockWrite->dwords[i], valRd = blockRead->dwords[i];
            if (valWrt != valRd) {
                ret = MIL1553_BC_WRT_DATA_RAM_ERROR_BAD_CHECK;
                goto END;
            }
        }
    }
END:
    // освобождаем буфер микропрограммы
    mil1553ud_freeBcRamBlock(blockWrite);
    mil1553ud_freeBcRamBlock(blockRead);
    return ret;
}

unsigned int extmil1553_rt_manSubaddresses(Mil1553ChannelFD cfd, unsigned int direction, unsigned int subadr_mask, unsigned int action_mask)
{
    unsigned int ret = GOOD_CODE;
    RT_BUF_MAN_EN rtBufMan;
    rtBufMan.direction = direction;
    rtBufMan.subaddress_mask = subadr_mask;  // меняем с 1 по 30 подадрес
    rtBufMan.action_mask = action_mask;  // включаем на приём с 1 по 30 подадрес
    ret = WRAPPER_IOCTL_RT_BUF_MAN_EN(cfd, &rtBufMan);
    return ret;
}

unsigned int extmil1553_rt_setAddress(Mil1553ChannelFD cfd, unsigned char address)
{
    unsigned int ret = GOOD_CODE;
    ret = WRAPPER_IOCTL_SET_ADDRESS(cfd, &address);
    return ret;
}

unsigned int extmil1553_rt_manSubadressTransmitBufferMode(Mil1553ChannelFD cfd, unsigned char subaddress, unsigned int mode)
{
    unsigned int ret = GOOD_CODE;
    RT_TR_MODE rt_tr_mode;
    rt_tr_mode.mode = mode;
    rt_tr_mode.subaddress = subaddress;
    ret = WRAPPER_IOCTL_SET_RT_TR_MODE(cfd, &rt_tr_mode);
    return ret;
}

unsigned int extmil1553_rt_manSubaddressTransmitBufferReady(Mil1553ChannelFD cfd, unsigned char subaddress, unsigned int cmd_buf)
{
    unsigned int ret = GOOD_CODE;
    RT_TR_MODE_BUF rdBuf;
    rdBuf.cmd_buf = cmd_buf;
    rdBuf.subaddress = subaddress;
    ret = WRAPPER_IOCTL_SET_RT_TR_BUF_READY(cfd, &rdBuf);
    return ret;
}

unsigned int extmil1553_rt_writeDataToSubaddressTransmit(Mil1553ChannelFD cfd, unsigned int num_buf, unsigned char subaddress, struct TRtSubaddressData data)
{
    unsigned int ret = GOOD_CODE;
    RT_TR_BUF_SUBADDR bufSub;
    bufSub.num_buf = num_buf;
    bufSub.subaddress = subaddress;
    memcpy(bufSub.data_words, data.mWords, data.mCount * 2);
    ret = WRAPPER_IOCTL_WR_BLOCK_BUF_SUBADR(cfd, &bufSub);
    return ret;
}

/*-------------------------------------------------------------------------------------------------------*/

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ДМА

unsigned int extmil1553ud_getCountDmaBlocksReadyRead(Mil1553ChannelFD cfd, unsigned int *codeError)
{
    unsigned int countBlocks = 0;  // кол-во блоков
    *codeError = WRAPPER_IOCTL_GET_NBLOCK_RAW_DMA(cfd, &countBlocks);
    int code = *codeError;
    if (code)
        return LIB_BAD_VALUE;  // ошибка
    else
        return countBlocks;
}

DMA_READ_BLOCK *_ext_mil1553ud_newDmaReadBlock(unsigned int countDmaBlocks) {
    DMA_READ_BLOCK *block = (DMA_READ_BLOCK *)malloc(sizeof(DMA_READ_BLOCK));
    block->countBlocks = countDmaBlocks;
    block->length = block->countBlocks * MIL_DMA_BLOCK_SIZE;
    block->data = (unsigned char *)malloc(block->length);
    return block;
}

void _ext_mil1553ud_freeDmaReadBlock(DMA_READ_BLOCK *block) {
    if (block != NULL) {
        if (block->data != NULL) {
            free(block->data);
        }
        free(block);
    }
}

DMA_READ_BLOCK *_ext_mil1553ud_readAllDmaBlocksReadyRead(Mil1553ChannelFD cfd, unsigned int *cntBlocks)
{
    DMA_READ_BLOCK *block = NULL;  // контейнер дма блоков
    unsigned int countBlocks = 0;  // кол-во блоков
    unsigned int codeError = 0;
    countBlocks = extmil1553ud_getCountDmaBlocksReadyRead(cfd, &codeError);
    *cntBlocks = countBlocks;
    if (countBlocks == 0) {
        return block;
    } else {
        if (codeError != 0) {
            return NULL;
        }
        block = mil1553ud_newDmaReadBlock(countBlocks);
        if (WRAPPER_IOCTL_READ_BLOCKS_RAW_DMA(cfd, block)) {  // ошибка
            mil1553ud_freeDmaReadBlock(block);
            return NULL;
        } else {
            return block;
        }
    }
}

/// \remark РАБОТА С КОЛЬЦЕВЫМ БУФЕРОМ ПРЕРЫВАНИЙ

unsigned int extmil1553ud_getCountInterruptBlocksReadyRead(Mil1553ChannelFD cfd)
{
    unsigned int countBlocks = 0;  // кол-во блоков
    if (WRAPPER_IOCTL_GET_COUNT_INTERRUPT_BLOCKS(cfd, &countBlocks))
        return LIB_BAD_VALUE;  // ошибка
    else
        return countBlocks;
}

unsigned int extmil1553ud_readInterruptBlocks(Mil1553ChannelFD cfd, INTERRUPT_BLOCK_BUFFER *block)
{
    unsigned int resutl = GOOD;
    resutl = WRAPPER_IOCTL_READ_INTERRUPT_BLOCKS(cfd, block);
    return resutl;
}

/// \remark РЕЖИМ КШ

unsigned int extmil1553ud_bcProgram(Mil1553ChannelFD cfd, unsigned int action)
{
    SADDR_DATA_BIT_MASK bitMask;  // битовая маска регистра
    unsigned int numChannel = cfd.channel;

    bitMask.daddr = MIL_REG(numChannel, BC_CONF_REG_PCI_CH);  // адрес регистра
    bitMask.mask = 1 << 15;  // изменяем только 15 бит - BCSTRT
    if (action == (unsigned int)BC_PROGRAM_START) {
        bitMask.data = 1 << 15;  // BCSTRT в 1
    } else {
        bitMask.data = 0;
    }
    if (WRAPPER_IOCTL_WRITE_REG_BIT_MASK(cfd, &bitMask))
        return LIB_BAD_VALUE;  // ошибка
    else
        return LIB_GOOD;
}

unsigned int extmil1553ud_bcSetStartInstructionAddress(Mil1553ChannelFD cfd, unsigned int startNumber)
{
    SADDR_DATA_BIT_MASK bitMask;  // битовая маска регистра
    unsigned int numChannel = cfd.channel;
    bitMask.daddr =
            MIL_REG(numChannel,
                    START_ADR_INSTR_PTR_CH);  // адрес регистра стартовой инструкции
    bitMask.mask = 0xFFF << 16;
    bitMask.data = (startNumber << 16) & (0xFFF << 16);
    if (WRAPPER_IOCTL_WRITE_REG_BIT_MASK(cfd, &bitMask))
        return LIB_BAD_VALUE;  // ошибка
    else
        return LIB_GOOD;
}

/// \remark РЕЖИМ КШ - РАБОТА С МИКРОПРОГРАММОЙ КШ

BC_RAM_BLOCK *_ext_mil1553ud_newBcRamBlock(unsigned int type,
                                         unsigned int sizeInDwords,
                                         unsigned int shiftInDwords) {
    BC_RAM_BLOCK *block = (BC_RAM_BLOCK *)malloc(sizeof(BC_RAM_BLOCK));
    block->type = type;
    block->length = sizeInDwords;
    block->shift = shiftInDwords;
    block->dwords = (unsigned int *)malloc(block->length * DWORD_SIZE);
    return block;
}

void _ext_mil1553ud_freeBcRamBlock(BC_RAM_BLOCK *block) {
    if (block != NULL) {
        if (block->dwords != NULL) {
            free(block->dwords);
        }
        free(block);
    }
}

/// \remark РАСШИФРОВКА БЛОКОВ ДМА

unsigned int _ext_mil1553ud_dmaBlock_getTypeTransaction(unsigned int typeDmaBlock,
                                                   unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 1) & 0xF;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 27) & 0xF;
    }
}

unsigned int _ext_mil1553ud_dmaBlock_isSuccessTransaction(unsigned int typeDmaBlock,
                                                     unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 28) & 0x1) ? MIL1553UD_TRUE : MIL1553UD_FALSE;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 26) & 0x1) ? MIL1553UD_FALSE : MIL1553UD_TRUE;
    }
}

unsigned int _ext_mil1553ud_dmaBlock_getFreeTimer(unsigned int typeDmaBlock,
                                             unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 1 * DWORD_SIZE);
        return *dword;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 4 * DWORD_SIZE);
        return *dword;
    }
}

unsigned int _ext_mil1553ud_dmaBlock_getCountServiceWords16(
        unsigned int typeDmaBlock, unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 11) & 0xF;
    }
}

unsigned int _ext_mil1553ud_dmaBlock_getCountDataWords16(unsigned int typeDmaBlock,
                                                    unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return (*dword >> 5) & 0x3F;
    }
}

unsigned int _ext_mil1553ud_dmaBlock_getActiveBus(unsigned int typeDmaBlock,
                                             unsigned char *dmaBlock) {
    if (MIL1553UD_BC_DMA_BLOCK == typeDmaBlock) {  // КШ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 15) & 0x1) ? MIL1553_BUS_B : MIL1553_BUS_A;
    } else {  // ОУ
        unsigned int *dword = (unsigned int *)(dmaBlock + 0 * DWORD_SIZE);
        return ((*dword >> 31) & 0x1) ? MIL1553_BUS_B : MIL1553_BUS_A;
    }
}

unsigned char *_ext_mil1553ud_dmaBlock_getPtrDataPart(unsigned char *dmaBlock) {
    return dmaBlock + MIL_DMA_BLOCK_SIZE / 2;
}

unsigned int _ext_mil1553ud_dmaBlock_tdModeSubaddress(unsigned char *dmaBlock) {
    return *((unsigned int *)dmaBlock) & 0x1F;
}
