#ifndef EXTMIL1553UD_MEU_PROTO_H
#define EXTMIL1553UD_MEU_PROTO_H

#ifdef __KERNEL__
#include <linux/types.h>
#else /* USER */
#include <stdint.h>
#include <stdbool.h>
#endif

#define USB_PKT_SIZE 512

#define MEU_MAGIC 0x4288

#pragma pack(push, 1)

/// Заголовок пакета протокола
typedef struct
{
    uint16_t magic; ///< Кодовое слово
    uint16_t size;  ///< Размер пакета
    uint16_t chan;  ///< Идентификатор канала
    uint16_t op;    ///< Команда
}
meu_header;

/// Простой ответ с кодом возврата
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
}
meu_simple_answer;

/// Запрос: команда "запись в регистр"
typedef struct
{
    meu_header header; ///< Заголовок
    uint64_t daddr; ///< адрес регистра (смещение в соотв. со спецификацией)
    uint32_t data; ///< значение регистра
}
meu_write_reg_request;


/// Структура битовой маски для команды "Запись в регистр заданных бит"
typedef struct
{
    uint64_t daddr; ///< адрес регистра (смещение в соотв. со спецификацией)
    uint32_t data; ///< значение регистра

    /**
     * @brief битовая маска
     *
     * 1 - бит записывается из data, 0 - бит остаётся неизменным
     */
    uint32_t mask;
}
meu_saddr_data_bit_mask;

/// Запрос: Запись в регистр заданных бит
typedef struct
{
    meu_header header; ///< Заголовок
    meu_saddr_data_bit_mask bitmask; ///< битовая маска
}
meu_write_reg_bit_mask_request;

/// Запрос: команда "чтение из регистра"
typedef struct
{
    meu_header header; ///< Заголовок
    uint64_t daddr; ///< адрес регистра (смещение в соотв. со спецификацией)
}
meu_read_reg_request;

/// Ответ: команда "чтение из регистра"
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
    uint32_t data; ///< значение регистра
}
meu_read_reg_answer;

/// Ответ: команда "чтение версии драйвера"
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
    uint32_t version; ///< Код версии
}
meu_version_driver_answer;

#define MEU_VERSION_DEVNAME_SIZE 30

/// информация о драйвере и устройстве
typedef struct
{
    uint32_t device_id; ///< идентификатор устройства
    uint32_t vendor_id; ///< вендор устройства
    uint32_t type; ///< тип устройства (кол-во каналов)
    uint8_t revision; ///< ревизия устройства
    char dev_name[ MEU_VERSION_DEVNAME_SIZE ]; ///< имя символьного устройства
    int32_t minor; ///< значение минора
    int32_t irq; ///< номер прерывания
    uint64_t size_dma; ///< размер ДМА буфера
    // void* addr_dma_virt; ///< виртуальный адрес ДМА буфера
    uint32_t pci_bars; ///< адрес bar-пространства
    // void* addr_bar_virt; ///< виртуальный адрес bar-пространства
}
meu_version_data;

/// Ответ: команда "информация о драйвере и устройстве"
typedef struct
{
    meu_header header;
    int64_t ret;
    meu_version_data version;
}
meu_version_answer;

/// Запрос: команда "смена режима"
typedef struct
{
    meu_header header; ///< Заголовок
    int32_t mode; ///< Режим
}
meu_switch_request;

/// Запрос: команда "установить режим ОУ"
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t subaddress; ///< подадрес
    uint32_t mode; ///< режим
}
meu_set_rttr_request;

/// Запрос: управление подадресами в режиме ОУ
typedef struct
{
    meu_header header; ///< Заголовок
    uint32_t direction; ///< направление (приём/передача)

    /**
     * @brief маскирование (выбор) по подадресам 1й бит - 1й подадрес и т.д.
     *
     * 1 - бит изменяем, 0 - бит не изменяем
     */
    uint32_t subaddress_mask;

    /**
     * @brief маскирование (выбор) действий по подадресам 1й бит - 1й подадрес и т.д.
     *
     * 1 - вкл, 0 - выкл
     */
    uint32_t action_mask;
}
meu_set_rt_buf_man_request;

/// размер блока ДМА (128 байт)
#define MEU_SIZE_RAW_DMA_BLOCK                  128

/// Запрос на чтение блоков DMA
typedef struct
{
    meu_header header; ///< Заголовок
    uint16_t n_blocks; ///< Количество блоков
}
meu_read_dma_request;

/// Ответ чтения блоков DMA
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
    uint32_t n_currentBlocks; ///< Количество блоков в полученной порции
    uint32_t n_nextBlocks; ///< Количество блоков в следующей порции
}
meu_read_dma_answer;

/// Запрос: команда "Установить адрес ОУ"
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t daddr; ///< адрес ОУ
}
meu_set_addr_request;

/// Запрос: команда "управление таймаутами таймеров"
typedef struct
{
    meu_header header; ///< Заголовок
    uint32_t tck; ///< значение RT_TCK
    uint32_t trck; ///< RT_TRCK
    uint32_t rcvck; ///< RT_RCVCK
}
meu_set_timer_request;

#define MEU_RTTR_SUBADDR_DATA_WORDS_COUNT 32

/// Запрос: команда "записать подадрес на отправку"
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t num_buf; ///< номер буфера
    uint8_t subaddress; ///< подадрес (1-30)
    uint16_t data_words[ MEU_RTTR_SUBADDR_DATA_WORDS_COUNT ]; ///< подадрес (32 слова данных)
}
meu_write_rttr_subaddr_request;


/// Запрос: команда "считать подадрес на отправку"
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t num_buf; ///< номер буфера
    uint8_t subaddress; ///< подадрес (1-30)
}
meu_read_rttr_subaddr_request;

/// Запрос: команда "считать подадрес на отправку"
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
    uint16_t data_words[ MEU_RTTR_SUBADDR_DATA_WORDS_COUNT ]; ///< подадрес (32 слова данных)
}
meu_read_rttr_subaddr_answer;

/// Запрос: команда чтения/записи блока данных в BC_RAM (область микропрограммы для КШ)
typedef struct
{
    meu_header header; ///< Заголовок

    /**
     @brief тип инструкций

     INSTR, OPERATION, DATA
    */
    uint32_t type;

    /**
     @brief смещение от начала буфера в 32-х разрядных словах

     INSTR - max 4095, OPERATION - max 4095, DATA - max 8191
    */
    uint32_t shift;

    uint32_t length; ///< размер поля dwords в 32-х разрядных словах

    //данные (массив 32-х разрядных слов)
}
meu_io_bc_ram_request;

#define MEU_RCV_SUBADDR_DATA_WORDS_COUNT 32

/// Запрос: команда "считать подадрес на отправку"
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t sa; ///< подадрес (1-30)
}
meu_read_rcv_subaddr_request;

/// Запрос: команда "считать подадрес на отправку"
typedef struct
{
    meu_header header; ///< Заголовок
    int64_t ret; ///< Код возврата
    uint16_t data_words[ MEU_RCV_SUBADDR_DATA_WORDS_COUNT ]; ///< слова данных
}
meu_read_rcv_subaddr_answer;

/// Запрос на чтение блоков прерываний
typedef struct
{
    meu_header header; ///< Заголовок
    uint16_t n_blocks; ///< Количество блоков
}
meu_read_intblocks_request;

/// \brief блок информации о прерывании
typedef struct
{
    /// \brief значение таймера в момент прерывани
    uint32_t free_timer_value;
    /// \brief маска прерываний (собственно по ней поймём какие в этот момент времени были прерывания)
    /// 0 - INT_HDAT, 1 - INT_QDAT, 2 - RT_INT_SADDR, 3 - RT_INT_MC_ERR,
    /// 4 - INT_BC, 5 - INT_FLASH, 6 - INT_DATA_CNT_EN,
    /// 7 - INT_TIMEOUT_ITVEN, 8 - INT_TIMEOUT_ABSEN
    uint32_t interrupt_ch;
    /// \brief регистр HW_STAT_REG1
    uint32_t hw_stat_reg1;
    /// \brief регистр HW_STAT_REG2
    uint32_t hw_stat_reg2;
}
meu_intblock_info;

#define MEU_BC_RAM_WRITE_SIZE_BLOCK 128


#define MEU_CONF_BLOCK_UPDATE_DESC_SIZE 64

/// \brief Структура блока обновления ПО lan1553ud
typedef struct
{
    uint8_t block_id; ///< [in] Индекс раздела обновления
    bool is_protected; ///< [out] Разрешено ли обновление в этот раздел
    uint8_t active_block_id; ///< [in] Индекс раздела обновления

    uint8_t mac_addr[6]; ///< мак-адрес

    uint8_t is_static_ip_enabled; ///< если false, адрес будет взят по dhcp. Если true - из поля ниже
    uint8_t ipv4_addr[4]; ///< адрес интерфейса

    uint8_t is_netmask_set; ///< если false, шлюз не учитывается
    uint8_t ipv4_netmask[4]; ///< сетевая маска

    uint8_t is_gateway_set; ///< если false, шлюз не учитывается
    uint8_t ipv4_gateway[4]; ///< шлюз

    uint64_t mil_serial_num; ///< Уникальный идентификатор устройства в серии (для различения нескольких usb)

    uint16_t server_port; ///< порт для внутреннего ПО mil-eth-server

    char update_desc[ MEU_CONF_BLOCK_UPDATE_DESC_SIZE ]; ///< строка с описанием обновления (ascii латиница)
}
meu_conf_block;


/// Запрос: команда записи секции настроек
typedef struct
{
    meu_header header; ///< Заголовок
    uint8_t block_id; ///< идентификатор блока
}
meu_io_conf_rd_request;

/// Запрос: установить активный загрузочный раздел
typedef meu_io_conf_rd_request meu_ext_set_current_block_request;


/// Запрос: подготовка обновления
typedef struct
{
    meu_header header; ///< Заголовок
    meu_conf_block conf; ///< секция настроек
}
meu_io_conf_wr_request;

typedef meu_io_conf_wr_request meu_fw_begin_request;


/// Запрос: команда записи секции настроек
typedef struct
{
    meu_header header; ///< Заголовок

    uint64_t ret; ///< Код возврата

    meu_conf_block conf; ///< секция настроек
}
meu_io_conf_result;

/// Ответ: статус обновления
typedef struct
{
    meu_header header; ///< Заголовок

    uint16_t stage; ///< Фаза обновления
    uint16_t percentComplete; ///< Прогресс в процентах
    int64_t ret; ///< Код возврата
}
meu_fw_status_answer;

/// Запрос: команда записи блока данных обновления
typedef struct
{
    meu_header header; ///< Заголовок

    uint32_t blockSize; ///< размер блока (в байтах)
    uint32_t blockOffset; ///< отступ блока (в байтах)
    uint32_t fileSize; ///< полный размер файла обновления (в байтах)
    //данные
}
meu_io_fw_request;

#define MEU_MAX_DATAGRAM_SIZE sizeof(meu_io_bc_ram_request) + MEU_BC_RAM_WRITE_SIZE_BLOCK * sizeof(unsigned int)

/// Нет операции (проверка соединения)
#define MEU_OP_NONE 0

/**
  @brief Запрос: количество каналов

  Идёт с помощью meu_header
*/
#define MEU_OP_CH_COUNT_REQ                                 1   // запрос кол-ва каналов

/**
  @brief Ответ: количество каналов

  Идёт с помощью meu_simple_answer
*/
#define MEU_OP_CH_COUNT_ANS                                 2   // запрос кол-ва каналов

// read/write regs
/**
  @brief Запрос: команда "запись в регистр"

  Позволяет записать значение в заданный регистр управления
  Идёт с помощью meu_write_reg_request
*/
#define MEU_IOCTL_WRITE_REG_REQ                             3            // запись в регистр

/**
  @brief Ответ: команда "запись в регистр"

  Выдаёт код возврата.
  Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_WRITE_REG_ANS                             4            // запись в регистр

/**
  @brief Запрос: команда "запись в регистр заданных бит"

  позволяет изменить отдельные биты в заданном регистре управления
  Идёт с помощью meu_write_reg_bit_mask_request
*/
#define MEU_IOCTL_WRITE_REG_BIT_MASK_REQ                    5   // запись в регистр заданных бит

/**
  @brief Ответ: команда "запись в регистр заданных бит"

  Выдаёт код возврата.
  Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_WRITE_REG_BIT_MASK_ANS                    6   // запись в регистр заданных бит

/**
  @brief Запрос: команда "чтение из регистра"

  позволяет прочитать значение из заданного регистра управления
  Идёт с помощью meu_read_reg_request
*/
#define MEU_IOCTL_READ_REG_REQ                              7            // чтение из регистра

/**
  @brief Ответ: команда "чтение из регистра"

  позволяет прочитать значение из заданного регистра управления
  Идёт с помощью meu_read_reg_answer
*/
#define MEU_IOCTL_READ_REG_ANS                              8            // чтение из регистра

// info command
/**
  @brief Запрос: команда "чтение подробной информации о плате и драйвере"

  позволяет считать подробную информацию о pci-плате
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_VERSION_REQ                               9               // получить информацию  о плате

/**
  @brief Ответ: команда "чтение подробной информации о плате и драйвере"

  позволяет считать подробную информацию о pci-плате
  Идёт с помощью meu_version_answer
*/
#define MEU_IOCTL_VERSION_ANS                              10               // получить информацию  о плате

/**
  @brief Запрос: команда "чтение версии драйвера"

  позволяет считать версию и дату драйвера
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_VERSION_DRIVER_REQ                       11          // получить информацию о драйвере

/**
  @brief Запрос: команда "чтение версии драйвера"

  позволяет считать версию и дату драйвера
  Идёт с помощью meu_version_driver_answer
*/
#define MEU_IOCTL_VERSION_DRIVER_ANS                       12          // получить информацию о драйвере

/*
 * Не поддерживается
 *
/// \brief команда "чтение информации о pci-локации платы"
/// позволяет считать информацию pci
#define MEU_IOCTL_PCI_LOCATION_REQ                         13          // получить информацию о pci-локации платы
#define MEU_IOCTL_PCI_LOCATION_ANS                         14          // получить информацию о pci-локации платы
*/

// dma
/**
  @brief Запрос: команда "сброс указателя dma канала"

  позволяет обнулить DMA_INDEX и программные указатели чтения/записи
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_CLEAR_DMA_REQ                            15                         // сброс указателя dma канала

/**
  @brief Ответ: команда "сброс указателя dma канала"

  Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_CLEAR_DMA_ANS                            16                         // сброс указателя dma канала

/**
  @brief Запрос: команда "включить dma устройства"

  разрешает работу DMA
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_ENABLE_DMA_REQ                           17                         // включить dma платы

/**
  @brief Ответ: команда "включить dma устройства"

  Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_ENABLE_DMA_ANS                           18                         // включить dma платы

/**
  @brief Запрос: команда "выключить dma устройства"

  запрещает работу DMA
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_DISABLE_DMA_REQ                          19                         // выключить dma платы

/**
  @brief Ответ: команда "выключить dma устройства"

  Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_DISABLE_DMA_ANS                          20                         // выключить dma платы

/**
  @brief Запрос: команда "получить количество 128 байтных блоков из ДМА"

  позволяет получить количество готовых для чтения блоков из буфера ДМА
  Идёт с помощью meu_header
*/
#define MEU_IOCTL_GET_NBLOCK_RAW_DMA_REQ                   21          // получить количество блоков из ДМА в сыром виде

/**
  @brief Ответ: команда "получить количество 128 байтных блоков из ДМА"

  Идёт с помощью meu_read_reg_answer
*/
#define MEU_IOCTL_GET_NBLOCK_RAW_DMA_ANS                   22          // получить количество блоков из ДМА в сыром виде

/**
  @brief Запрос: команда "считать заданное количество 128 байтных из ДМА"

  позволяет получить данные (блоки) из буфера ДМА
  Идёт с помощью meu_read_dma_request
*/
#define MEU_IOCTL_READ_BLOCKS_RAW_DMA_REQ                  23        // считать заданное кол-во блоков из ДМА

/**
  @brief Ответ: команда "считать заданное количество 128 байтных из ДМА"

  Идёт с помощью meu_simple_answer (после которого блоки данных)
  где в коде возврата указано количество блоков.

  В случае кода ошибки, размер датаграммы равен meu_simple_answer
  В случае успеха, размер датаграммы равен meu_simple_answer + 128 * n_blocks
*/
#define MEU_IOCTL_READ_BLOCKS_RAW_DMA_ANS                  24        // считать заданное кол-во блоков из ДМА

// irq
/// \brief команда "управление прерываниями INTERRUPT_MASK"
#define MEU_IOCTL_MANAGE_INTERRUPT_REQ                     25         // управление прерываниями INTERRUPT_MASK
#define MEU_IOCTL_MANAGE_INTERRUPT_ANS                     26         // управление прерываниями INTERRUPT_MASK

/// \brief команда "управление прерываниями по передаче из подадреса"
#define MEU_IOCTL_MAN_IRQ_SUBADDRESS_TR_REQ                27  // управление прерываниями по передаче из подадреса
#define MEU_IOCTL_MAN_IRQ_SUBADDRESS_TR_ANS                28  // управление прерываниями по передаче из подадреса

/// \brief команда "управление прерываниями по приёму в подадрес"
#define MEU_IOCTL_MAN_IRQ_SUBADDRESS_RCV_REQ               29  // управление прерываниями по приёму в подадрес
#define MEU_IOCTL_MAN_IRQ_SUBADDRESS_RCV_ANS               30  // управление прерываниями по приёму в подадрес

// mode
/**
 @brief Запрос: команда "выбор режима работы канала"

 Идёт с помощью meu_switch_request
 */
#define MEU_IOCTL_SWITCH_MODE_REQ                          31          // выбор режима работы канала

/**
 @brief Ответ: команда "выбор режима работы канала"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SWITCH_MODE_ANS                          32          // выбор режима работы канала


/**
 @brief Запрос: команда "выбор шины канала"

 Идёт с помощью meu_switch_request
*/
#define MEU_IOCTL_SWITCH_BUS_REQ                           33          // выбор шины канала

/**
 @brief Ответ: команда "выбор режима работы канала"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SWITCH_BUS_ANS                           34          // выбор шины канала


/**
 @brief Запрос: команда "разрешить работу канала"

 Идёт с помощью meu_switch_request
 */
#define MEU_IOCTL_DEV_CHANNEL_WORK_REQ                     35          // вкл/выкл канал

/**
 @brief Ответ: команда "разрешить работу канала"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_DEV_CHANNEL_WORK_ANS                     36          // вкл/выкл канал

// term dev
/**
 @brief Запрос: команда "установить режим ОУ"

 Идёт с помощью meu_set_rttr_request
 */
#define MEU_IOCTL_SET_RT_TR_MODE_REQ                       37            // установить режим ОУ

/**
 @brief Ответ: команда "установить режим ОУ"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SET_RT_TR_MODE_ANS                       38            // установить режим ОУ

/**
 @brief Запрос: команда "установить готовность буфера ОУ"

 Идёт с помощью meu_set_rttr_request
 */
#define MEU_IOCTL_SET_RT_TR_BUF_READY_REQ                  39        // установить готовность буфера ОУ

/**
 @brief Ответ: команда "установить готовность буфера ОУ"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SET_RT_TR_BUF_READY_ANS                  40        // установить готовность буфера ОУ

/**
 @brief Запрос: команда "управление разрешением буферов"

 Идёт с помощью meu_set_rt_buf_man_request
 */
#define MEU_IOCTL_RT_BUF_MAN_EN_REQ                        41         // установка разрешение буферов ОУ

/**
 @brief Ответ: команда "управление разрешением буферов"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_RT_BUF_MAN_EN_ANS                        42         // установка разрешение буферов ОУ

// channel
/**
 @brief Запрос: команда "установить адрес ОУ"

 Идёт с помощью meu_set_addr_request
 */
#define MEU_IOCTL_SET_ADDRESS_REQ                          43         // установить адрес ОУ

/**
 @brief Ответ: команда "управление разрешением буферов"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SET_ADDRESS_ANS                          44         // установить адрес ОУ

/**
 @brief Запрос: команда "установить таймауты ОУ"

 Идёт с помощью meu_set_timer_request
 */
#define MEU_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI_REQ          45         // set timer and timeout

/**
 @brief Ответ: команда "установить таймауты ОУ"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SET_TIMER_RTBM_CONF_REG_PCI_ANS          46         // set timer and timeout

/**
 @brief Запрос: команда "установить таймауты КШ"

 Идёт с помощью meu_set_timer_request
 */
#define MEU_IOCTL_SET_TIMER_BC_CONF_REG_PCI_REQ            47         // set timer and timeout

/**
 @brief Ответ: команда "установить таймауты КШ"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_SET_TIMER_BC_CONF_REG_PCI_ANS            48         // set timer and timeout

// paddr and buf
/**
 @brief Запрос: команда "записать подадрес на отправку"

 Идёт с помощью meu_write_rttr_subaddr_request
 */
#define MEU_IOCTL_WR_BLOCK_BUF_SUBADR_REQ                  49     // записать поддадрес

/**
 @brief Ответ: команда "установить таймауты КШ"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_WR_BLOCK_BUF_SUBADR_ANS                  50     // записать поддадрес

/// \brief команда "прочитать подадрес на отправку"
#define MEU_IOCTL_RD_BLOCK_BUF_SUBADR_REQ                  51     // прочитать поддадрес
#define MEU_IOCTL_RD_BLOCK_BUF_SUBADR_ANS                  52     // прочитать поддадрес

// bc_ram
/**
 @brief Запрос: команда "запись блока в BC RAM"

 Идёт с помощью meu_io_bc_ram_request
 После которого идёт указанное количество полезных данных
 */
#define MEU_IOCTL_WRITE_BC_RAM_REQ                         53          // запись в BC ram

/**
 @brief Ответ: команда "запись блока в BC RAM"

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_WRITE_BC_RAM_ANS                         54          // запись в BC ram

/**
 @brief Запрос: команда "чтение блока из BC RAM"

 Идёт с помощью meu_io_bc_ram_request
*/
#define MEU_IOCTL_READ_BC_RAM_REQ                          55          // чтение из BC ram

/**
 @brief Ответ: команда "чтение блока из BC RAM"

 Идёт с помощью meu_simple_answer
 После которого идёт указанное количество полезных данных
*/
#define MEU_IOCTL_READ_BC_RAM_ANS                          56          // чтение из BC ram

// interrupt ring buffer
/**
 @brief Запрос: получить кол-во блоков с информацией о возникших прерываниях из кольцевого буфера

 идёт с помощью meu_header
 */
#define MEU_IOCTL_GET_COUNT_INTERRUPT_BLOCKS_REQ           57          // получить кол-во блоков с информацией о возникших прерываниях

/**
  @brief Ответ: получить кол-во блоков с информацией о возникших прерываниях из кольцевого буфера

  идёт с помощью meu_read_reg_answer
 */
#define MEU_IOCTL_GET_COUNT_INTERRUPT_BLOCKS_ANS           58          // получить кол-во блоков с информацией о возникших прерываниях

/**
 @brief Запрос: считать заданное кол-во блоков из кольцевого буфера

 идёт с помощью meu_read_intblocks_request
 */
#define MEU_IOCTL_READ_INTERRUPT_BLOCKS_REQ                59  // считать заданное кол-во блоков из кольцевого буфера

/**
 @brief Ответ: считать заданное кол-во блоков из кольцевого буфера

 идёт с помощью meu_read_intblocks_answer
*/
#define MEU_IOCTL_READ_INTERRUPT_BLOCKS_ANS                60  // считать заданное кол-во блоков из кольцевого буфера

/**
 @brief Запрос: считать входной поддадрес

идёт с помощью meu_read_rcv_subaddr_request
*/
#define MEU_IOCTL_GET_RCV_SUBADR_DATA_REQ                  61         // считать входной поддадрес

/**
 @brief Ответ: считать входной поддадрес

 идёт с помощью meu_read_rcv_subaddr_answer
*/
#define MEU_IOCTL_GET_RCV_SUBADR_DATA_ANS                  62         // считать входной поддадрес

/**
 @brief Запрос: начать процесс обновления

 Идёт с помощью meu_fw_begin_request
*/
#define MEU_IOCTL_FW_BEGIN_REQ                             63         // обновление: начать

/**
 @brief Ответ: начать процесс обновления

 Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_FW_BEGIN_ANS                             64         // обновление: начать

/**
 @brief Запрос: загрузить обновление

 Идёт с помощью meu_io_fw_request
 После которого идёт указанное количество полезных данных
*/
#define MEU_IOCTL_FW_LOAD_REQ                              65         // обновление: загрузить

/**
 @brief Ответ: загрузить обновление

 Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_FW_LOAD_ANS                              66         // обновление: загрузить

/**
 @brief Запрос: прошить обновление во flash

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_FW_UPDATE_APPLY_REQ                            67         // обновление: прошить обновление во flash

/**
 @brief Ответ: прошить обновление во flash

 Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_FW_UPDATE_APPLY_ANS                            68         // обновление: прошить обновление во flash

/**
 @brief Запрос: статус обновления

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_FW_STATUS_REQ                            69         // обновление: статус

/**
 @brief Ответ: статус обновления

 Идёт с помощью meu_fw_status_answer
*/
#define MEU_IOCTL_FW_STATUS_ANS                            70         // обновление: статус

/**
 @brief Запрос: загрузить количество блоков обновления

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_FW_COUNT_REQ                            71         // обновление: количество блоков обновления

/**
 @brief Ответ: загрузить количество блоков обновления

 Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_FW_COUNT_ANS                            72         // обновление: количество блоков обновления

/**
 * \brief Запрос: установить настройки ПО lan1553ud

 Идёт с помощью meu_io_conf_wr_request
 */
#define MEU_IOCTL_FW_WR_SETTINGS_REQ                         73     // обновление: установить настройки ПО lan1553ud

/**
 * \brief Ответ: установить настройки ПО lan1553ud

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_FW_WR_SETTINGS_ANS                         74     // обновление: установить настройки ПО lan1553ud

/**
 * \brief Запрос: получить настройки ПО lan1553ud

 Идёт с помощью meu_io_conf_rd_request
 */
#define MEU_IOCTL_FW_RD_SETTINGS_REQ                         75     // обновление: получить настройки ПО lan1553ud

/**
 * \brief Ответ: получить настройки ПО lan1553ud

 Идёт с помощью meu_io_conf_result
 */
#define MEU_IOCTL_FW_RD_SETTINGS_ANS                         76     // обновление: получить настройки ПО lan1553ud

/**
 * \brief Запрос: перезагрузка внешнего устройства

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_EXT_REBOOT_REQ                             77     // перезагрузка внешнего устройства

/**
 * \brief Ответ: перезагрузка внешнего устройства

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_EXT_REBOOT_ANS                             78     // перезагрузка внешнего устройства

/**
 * \brief Запрос: перезагрузка внешнего устройства

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_EXT_GET_CURRENT_REQ                        79     // получить индекс активного раздела lan1553ud

/**
 * \brief Ответ: получить индекс активного раздела lan1553ud

 Идёт с помощью meu_simple_answer
 */
#define MEU_IOCTL_EXT_GET_CURRENT_ANS                        80     // получить индекс активного раздела lan1553ud

/**
 @brief Запрос: отмена обновления

 Идёт с помощью meu_header
*/
#define MEU_IOCTL_FW_UPDATE_CANCEL_REQ                       81         // обновление: отмена

/**
 @brief Ответ: отмена обновления

 Идёт с помощью meu_simple_answer
*/
#define MEU_IOCTL_FW_UPDATE_CANCEL_ANS                       82         // обновление: отмена
/*
 IOCTL_GET_NUMBER_CH
 Идёт на стороне хоста, без сетевого обмена
*/

#pragma pack(pop)


#endif // EXTMIL1553UD_MEU_PROTO_H
